@@ -4,16 +4,24 @@
 
 package Convert::ASN1;
 
-# $Id: ASN1.pm,v 1.20 2002/01/22 11:24:28 gbarr Exp $
+# $Id: ASN1.pm,v 1.27 2003/05/12 17:45:57 gbarr Exp $
 
 use 5.004;
 use strict;
 use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS @opParts @opName $AUTOLOAD);
 use Exporter;
 
+use constant CHECK_UTF8 => $] > 5.007;
+
 BEGIN {
+
+  if (CHECK_UTF8) {
+    require Encode;
+    require utf8;
+  }
+
   @ISA = qw(Exporter);
-  $VERSION = '0.15';
+  $VERSION = '0.17';
 
   %EXPORT_TAGS = (
     io    => [qw(asn_recv asn_send asn_read asn_write asn_get asn_ready)],
@@ -34,7 +42,7 @@ BEGIN {
   $EXPORT_TAGS{all} = \@EXPORT_OK;
 
   @opParts = qw(
-    cTAG cTYPE cVAR cLOOP cOPT cCHILD
+    cTAG cTYPE cVAR cLOOP cOPT cCHILD cDEFINE
   );
 
   @opName = qw(
@@ -101,7 +109,14 @@ sub new {
 sub configure {
   my $self = shift;
   my %opt = @_;
-  
+
+  $self->{options}{encoding} = uc($opt{encoding} || 'BER');
+
+  unless ($self->{options}{encoding} =~ /^[BD]ER$/) {
+    require Carp;
+    Carp::croak("Unsupported encoding format '$opt{encoding}'");
+  }
+
   for my $type (qw(encode decode)) {
     if (exists $opt{$type}) {
       while(my($what,$value) = each %{$opt{$type}}) {
@@ -128,12 +143,21 @@ sub prepare {
   my $asn  = shift;
 
   $self = $self->new unless ref($self);
-
-  my $tree = Convert::ASN1::parser::parse($asn);
+  my $tree;
+  if( ref($asn) eq 'GLOB' ){
+    local $/ = undef;
+    my $txt = <$asn>;
+    $tree = Convert::ASN1::parser::parse($txt);
+  } else {
+    $tree = Convert::ASN1::parser::parse($asn);
+  }
 
   unless ($tree) {
     $self->{error} = $@;
     return;
+    ### If $self has been set to a new object, not returning
+    ### this object here will destroy the object, so the caller
+    ### won't be able to get at the error.
   }
 
   $self->{tree} = _pack_struct($tree);
@@ -141,6 +165,27 @@ sub prepare {
   $self;
 }
 
+sub prepare_file {
+  my $self = shift;
+  my $asnp = shift;
+
+  local *ASN;
+  open( ASN, $asnp )
+      or do{ $self->{error} = $@; return; };
+  my $ret = $self->prepare( \*ASN );
+  close( ASN );
+  $ret;
+}
+
+sub registeroid {
+  my $self = shift;
+  my $oid  = shift;
+  my $handler = shift;
+
+  $self->{options}{oidtable}{$oid}=$handler;
+  $self->{oidtable}{$oid}=$handler;
+}
+
 # In XS the will convert the tree between perl and C structs
 
 sub _pack_struct { $_[0] }
@@ -197,14 +242,30 @@ sub asn_encode_length {
 
 sub decode {
   my $self  = shift;
-  my $stash = {};
 
   local $SIG{__DIE__};
-  eval { _decode($self->{options}, $self->{script}, $stash, 0, length $_[0], undef, [], $_[0]); $stash }
-  or do {
-    $self->{'error'} = $@;
-    undef;
+  my $ret = eval { 
+    my (%stash, $result);
+    my $script = $self->{script};
+    my $stash = (1 == @$script && !$self->{script}[0][cVAR]) ? \$result : ($result=\%stash);
+
+    _decode(
+	$self->{options},
+	$script,
+	$stash,
+	0,
+	length $_[0], 
+	undef,
+	[],
+	$_[0]);
+
+    $result;
   };
+  if ($@) {
+    $self->{'error'} = $@;
+    return undef;
+  }
+  $ret;
 }
 
 
